﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  internal class M5CreationParameters {
    private readonly ISplitType Impurity1;
    private readonly IPruningType Pruningtype1;
    private readonly ILeafType<IRegressionModel> LeafType1;
    private readonly int MinLeafSize1;
    private readonly IRegressionProblemData ProblemData1;
    private readonly IRandom Random1;
    private readonly ResultCollection Results1;
    public ISplitType Split {
      get { return Impurity1; }
    }
    public IPruningType Pruningtype {
      get { return Pruningtype1; }
    }
    public ILeafType<IRegressionModel> LeafType {
      get { return LeafType1; }
    }
    public int MinLeafSize {
      get { return MinLeafSize1; }
    }
    private IRegressionProblemData ProblemData {
      get { return ProblemData1; }
    }
    public IRandom Random {
      get { return Random1; }
    }
    public ResultCollection Results {
      get { return Results1; }
    }

    public ILeafType<IRegressionModel> PruningLeaf {
      get { return Pruningtype.ModelType(LeafType); }
    }
    public IEnumerable<string> AllowedInputVariables {
      get { return ProblemData.AllowedInputVariables; }
    }
    public string TargetVariable {
      get { return ProblemData.TargetVariable; }
    }
    public IDataset Data {
      get { return ProblemData.Dataset; }
    }

    public M5CreationParameters(IPruningType pruning, int minleafSize, ILeafType<IRegressionModel> modeltype,
      IRegressionProblemData problemData, IRandom random, ISplitType split, ResultCollection results) {
      Impurity1 = split;
      Pruningtype1 = pruning;
      ProblemData1 = problemData;
      Random1 = random;
      LeafType1 = modeltype;
      Results1 = results;
      var pruningLeaf = pruning.ModelType(LeafType);
      MinLeafSize1 = Math.Max(pruningLeaf == null ? 0 : pruningLeaf.MinLeafSize(problemData), Math.Max(minleafSize, modeltype.MinLeafSize(problemData)));
    }
  }
}